<?php

namespace App\Entity;

use DateTimeInterface;
use Doctrine\DBAL\Types\Types;
use Doctrine\ORM\Mapping as ORM;
use Gedmo\Mapping\Annotation as Gedmo;
use JsonSerializable;

/**
 * Implementation for the base implementation of an entity: id, creation and last update date and time
 */
trait TEntityBase {
    /**
     * @var int|null The internal id
     *
     * @noinspection PhpPropertyNamingConventionInspection
     */
    #[ORM\Id]
    #[ORM\GeneratedValue]
    #[ORM\Column]
    private ?int $id = null;

    /**
     * @var DateTimeInterface|null The creation date and time
     */
    #[ORM\Column(type: Types::DATETIME_IMMUTABLE, nullable: false)]
    #[Gedmo\Timestampable(on: 'create')]
    private ?DateTimeInterface $creation = null;
    /**
     * @var DateTimeInterface|null The last modification date and time
     */
    #[ORM\Column(type: Types::DATETIME_IMMUTABLE, nullable: false)]
    #[Gedmo\Timestampable(on: 'update')]
    private ?DateTimeInterface $lastModification = null;

    /**
     * Les données du trait qui doivent être inclus dans le JSON
     *
     * @return array Les données du trait qui doivent être inclus dans le JSON
     *
     * @see          JsonSerializable::jsonSerialize()
     *
     * @noinspection PhpMethodNamingConventionInspection
     */
    protected final function TEntityBase__jsonSerialize (): array {
        return [
            'id'               => $this->getId(),
            'creation'         => $this->getCreation()?->format(DateTimeInterface::RFC3339),
            'lastModification' => $this->getLastModification()?->format(DateTimeInterface::RFC3339),
        ];
    }

    /**
     * The internal id
     *
     * @return int|null The internal id
     */
    public function getId (): ?int {
        return $this->id;
    }

    /**
     * The creation date and time
     *
     * @return DateTimeInterface|null The creation date and time
     */
    public function getCreation (): ?DateTimeInterface {
        return $this->creation;
    }

    /**
     * The last modification date and time
     *
     * @return DateTimeInterface|null The last modification date and time
     */
    public function getLastModification (): ?DateTimeInterface {
        return $this->lastModification;
    }
}