<?php

namespace App\Form;

use App\Entity\User;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\CheckboxType;
use Symfony\Component\Form\Extension\Core\Type\PasswordType;
use Symfony\Component\Form\Extension\Core\Type\RepeatedType;
use Symfony\Component\Form\Extension\Core\Type\SubmitType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\Validator\Constraints\IsTrue;
use Symfony\Component\Validator\Constraints\Length;
use Symfony\Component\Validator\Constraints\NotBlank;

/**
 * The form for user registration (sign up)
 */
class SignUpFormType extends AbstractType {
    /**
     * @inheritDoc
     */
    public function configureOptions (OptionsResolver $resolver): void {
        $resolver->setDefaults(
            [
                'data_class' => User::class,
            ]
        );
    }

    /**
     * @inheritDoc
     */
    public function buildForm (FormBuilderInterface $builder, array $options): void {
        $builder
            ->add('email', null, [
                'label' => 'fields.email.label',
            ])
            ->add('name', null, [
                'label' => 'fields.name',
            ])
            ->add('newPassword', RepeatedType::class, [
                'type'            => PasswordType::class,
                'mapped'          => false,
                'attr'            => ['autocomplete' => 'new-password'],
                'invalid_message' => 'fields.password.errorMismatch',
                'constraints'     => [
                    new NotBlank(
                        [
                            'message' => 'fields.password.errorEmpty',
                        ]
                    ),
                    new Length(
                        [
                            'min'        => 6,
                            'minMessage' => 'fields.password.errorLength',
                            'max'        => 4096,           // max length allowed by Symfony for security reasons
                        ]
                    ),
                ],
                'first_options'   => [
                    'label' => 'fields.password.label',
                ],
                'second_options'  => [
                    'label' => 'fields.password.confirmationLabel',
                ],
            ])
            ->add('agreeTerms', CheckboxType::class, [
                'mapped'      => false,
                'label'       => 'fields.agreeTerms.label',
                'constraints' => [
                    new IsTrue(
                        [
                            'message' => 'fields.agreeTerms.errorNotTrue',
                        ]
                    ),
                ],
            ])
            ->add('submit', SubmitType::class, [
                'label' => 'fields.signUp',
            ]);
    }
}
